@@ -48,7 +48,7 @@ ExpoScreen::dndInit (CompAction         *action,
 		     CompAction::State  state,
 		     CompOption::Vector &options)
 {
-    if (expoMode)
+    if (expoMode && expoCam == 1.0f)
     {
 	dndState = DnDStart;
 	action->setState (action->state () | CompAction::StateTermButton);
@@ -65,7 +65,7 @@ ExpoScreen::dndFini (CompAction         *action,
 		     CompAction::State  state,
 		     CompOption::Vector &options)
 {
-    if (dndState == DnDDuring || dndState == DnDStart)
+    if (expoCam == 1.0f && (dndState == DnDDuring || dndState == DnDStart))
     {
 	if (dndWindow)
 	    finishWindowMovement ();
@@ -141,10 +141,13 @@ ExpoScreen::termExpo (CompAction         *action,
     if (dndState != DnDNone)
 	dndFini (action, state, options);
 
-    if (state & CompAction::StateCancel)
-	vpUpdateMode = VPUpdatePrevious;
-    else
-	vpUpdateMode = VPUpdateMouseOver;
+    if (expoCam == 1.0f)
+    {
+	if (state & CompAction::StateCancel)
+	    vpUpdateMode = VPUpdatePrevious;
+	else
+	    vpUpdateMode = VPUpdateMouseOver;
+    }
 
     dndState  = DnDNone;
     dndWindow = NULL;
@@ -324,6 +327,7 @@ ExpoScreen::handleEvent (XEvent *event)
 
 	case ButtonPress:
 	    if (expoMode			    &&
+		expoCam == 1.0f			    &&
 		event->xbutton.button == Button1    &&
 		event->xbutton.root   == screen->root ())
 	    {
@@ -354,6 +358,7 @@ ExpoScreen::handleEvent (XEvent *event)
 
 	case ButtonRelease:
 	    if (expoMode			    &&
+		expoCam == 1.0f			    &&
 		event->xbutton.button == Button1    &&
 		event->xbutton.root   == screen->root ())
 	    {
@@ -605,7 +610,7 @@ ExpoScreen::donePaint ()
 			       CompWindowGrabMoveMask |
 			       CompWindowGrabButtonMask);
 
-		screen->updateGrab (grabIndex, dragCursor);
+		screen->updateGrab (grabIndex, screen->cursorCache (XC_fleur));
 
 		w->raise ();
 		w->moveInputFocusTo ();
@@ -778,10 +783,10 @@ ExpoScreen::paintWall (const GLScreenPaintAttrib &attrib,
 
     /* Zoom animation stuff */
     /* camera position for the selected viewport */
-    GLVector vpCamPos (0, 0, 0, 0);
+    GLVector vpCamPos (0, 0, 0, 1);
 
     /* camera position during expo mode */
-    GLVector expoCamPos (0, 0, 0, 0);
+    GLVector expoCamPos (0, 0, 0, 1);
 
     float sx = screen->width ()  / static_cast <float> (output->width ());
     float sy = screen->height () / static_cast <float> (output->height ());
@@ -1573,8 +1578,6 @@ ExpoScreen::ExpoScreen (CompScreen *s) :
     upKey    = XKeysymToKeycode (s->dpy (), XStringToKeysym ("Up"));
     downKey  = XKeysymToKeycode (s->dpy (), XStringToKeysym ("Down"));
 
-    dragCursor = XCreateFontCursor (screen->dpy (), XC_fleur);
-
     EXPOINITBIND (ExpoKey, doExpo);
     EXPOTERMBIND (ExpoKey, termExpo);
     EXPOINITBIND (ExpoButton, doExpo);
@@ -1598,11 +1601,6 @@ ExpoScreen::ExpoScreen (CompScreen *s) :
 						     GL_RGBA, GL_UNSIGNED_BYTE);
 }
 
-ExpoScreen::~ExpoScreen ()
-{
-    if (dragCursor != None)
-	XFreeCursor (screen->dpy (), dragCursor);
-}
 
 ExpoWindow::ExpoWindow (CompWindow *w) :
     PluginClassHandler<ExpoWindow, CompWindow> (w),
