@@ -61,21 +61,20 @@ enum
     GWD_SETTINGS_IMPL_PROPERTY_INACTIVE_SHADOW = 2,
     GWD_SETTINGS_IMPL_PROPERTY_USE_TOOLTIPS = 3,
     GWD_SETTINGS_IMPL_PROPERTY_DRAGGABLE_BORDER_WIDTH = 4,
-    GWD_SETTINGS_IMPL_PROPERTY_ATTACH_MODAL_DIALOGS = 5,
-    GWD_SETTINGS_IMPL_PROPERTY_BLUR_CHANGED = 6,
-    GWD_SETTINGS_IMPL_PROPERTY_METACITY_THEME = 7,
-    GWD_SETTINGS_IMPL_PROPERTY_ACTIVE_OPACITY = 8,
-    GWD_SETTINGS_IMPL_PROPERTY_INACTIVE_OPACITY = 9,
-    GWD_SETTINGS_IMPL_PROPERTY_ACTIVE_SHADE_OPACITY = 10,
-    GWD_SETTINGS_IMPL_PROPERTY_INACTIVE_SHADE_OPACITY = 11,
-    GWD_SETTINGS_IMPL_PROPERTY_BUTTON_LAYOUT = 12,
-    GWD_SETTINGS_IMPL_PROPERTY_TITLEBAR_ACTION_DOUBLE_CLICK = 13,
-    GWD_SETTINGS_IMPL_PROPERTY_TITLEBAR_ACTION_MIDDLE_CLICK = 14,
-    GWD_SETTINGS_IMPL_PROPERTY_TITLEBAR_ACTION_RIGHT_CLICK = 15,
-    GWD_SETTINGS_IMPL_PROPERTY_MOUSE_WHEEL_ACTION = 16,
-    GWD_SETTINGS_IMPL_PROPERTY_TITLEBAR_FONT = 17,
-    GWD_SETTINGS_IMPL_PROPERTY_CMDLINE_OPTIONS = 18,
-    GWD_SETTINGS_IMPL_PROPERTY_SETTINGS_NOTIFIED = 19
+    GWD_SETTINGS_IMPL_PROPERTY_BLUR_CHANGED = 5,
+    GWD_SETTINGS_IMPL_PROPERTY_METACITY_THEME = 6,
+    GWD_SETTINGS_IMPL_PROPERTY_ACTIVE_OPACITY = 7,
+    GWD_SETTINGS_IMPL_PROPERTY_INACTIVE_OPACITY = 8,
+    GWD_SETTINGS_IMPL_PROPERTY_ACTIVE_SHADE_OPACITY = 9,
+    GWD_SETTINGS_IMPL_PROPERTY_INACTIVE_SHADE_OPACITY = 10,
+    GWD_SETTINGS_IMPL_PROPERTY_BUTTON_LAYOUT = 11,
+    GWD_SETTINGS_IMPL_PROPERTY_TITLEBAR_ACTION_DOUBLE_CLICK = 12,
+    GWD_SETTINGS_IMPL_PROPERTY_TITLEBAR_ACTION_MIDDLE_CLICK = 13,
+    GWD_SETTINGS_IMPL_PROPERTY_TITLEBAR_ACTION_RIGHT_CLICK = 14,
+    GWD_SETTINGS_IMPL_PROPERTY_MOUSE_WHEEL_ACTION = 15,
+    GWD_SETTINGS_IMPL_PROPERTY_TITLEBAR_FONT = 16,
+    GWD_SETTINGS_IMPL_PROPERTY_CMDLINE_OPTIONS = 17,
+    GWD_SETTINGS_IMPL_PROPERTY_SETTINGS_NOTIFIED = 18
 };
 
 enum
@@ -92,7 +91,6 @@ typedef struct _GWDSettingsImplPrivate
     decor_shadow_options_t inactive_shadow;
     gboolean		   use_tooltips;
     gint		   draggable_border_width;
-    gboolean		   attach_modal_dialogs;
     gint		   blur_type;
     gchar		   *metacity_theme;
     gdouble		   metacity_active_opacity;
@@ -260,24 +258,6 @@ gwd_settings_draggable_border_width_changed (GWDSettingsWritable *settings,
 	return FALSE;
 }
 
-static gboolean
-gwd_settings_attach_modal_dialogs_changed (GWDSettingsWritable *settings,
-					   gboolean            attach_modal_dialogs)
-{
-    GWDSettingsImpl        *settings_impl = GWD_SETTINGS_IMPL (settings);
-    GWDSettingsImplPrivate *priv = GET_PRIVATE (settings_impl);
-
-    if (priv->attach_modal_dialogs != attach_modal_dialogs)
-    {
-	priv->attach_modal_dialogs = attach_modal_dialogs;
-	append_to_notify_funcs (settings_impl, gwd_settings_notified_update_decorations);
-	release_notify_funcs (settings_impl);
-	return TRUE;
-    }
-    else
-	return FALSE;
-}
-
 static gboolean
 gwd_settings_blur_changed (GWDSettingsWritable *settings,
 			   const gchar         *type)
@@ -554,7 +534,6 @@ gwd_settings_writable_interface_init (GWDSettingsWritableInterface *interface)
     interface->shadow_property_changed = gwd_settings_shadow_property_changed;
     interface->use_tooltips_changed = gwd_settings_use_tooltips_changed;
     interface->draggable_border_width_changed = gwd_settings_draggable_border_width_changed;
-    interface->attach_modal_dialogs_changed = gwd_settings_attach_modal_dialogs_changed;
     interface->blur_changed = gwd_settings_blur_changed;
     interface->metacity_theme_changed = gwd_settings_metacity_theme_changed;
     interface->opacity_changed = gwd_settings_opacity_changed;
@@ -659,9 +638,6 @@ gwd_settings_get_property (GObject    *object,
 	case GWD_SETTINGS_IMPL_PROPERTY_DRAGGABLE_BORDER_WIDTH:
 	    g_value_set_int (value, priv->draggable_border_width);
 	    break;
-	case GWD_SETTINGS_IMPL_PROPERTY_ATTACH_MODAL_DIALOGS:
-	    g_value_set_boolean (value, priv->attach_modal_dialogs);
-	    break;
 	case GWD_SETTINGS_IMPL_PROPERTY_BLUR_CHANGED:
 	    g_value_set_int (value, priv->blur_type);
 	    break;
@@ -728,9 +704,6 @@ gwd_settings_impl_class_init (GWDSettingsImplClass *klass)
     g_object_class_override_property (object_class,
 				      GWD_SETTINGS_IMPL_PROPERTY_DRAGGABLE_BORDER_WIDTH,
 				      "draggable-border-width");
-    g_object_class_override_property (object_class,
-				      GWD_SETTINGS_IMPL_PROPERTY_ATTACH_MODAL_DIALOGS,
-				      "attach-modal-dialogs");
     g_object_class_override_property (object_class,
 				      GWD_SETTINGS_IMPL_PROPERTY_BLUR_CHANGED,
 				      "blur");
@@ -807,7 +780,6 @@ static void gwd_settings_impl_init (GWDSettingsImpl *self)
     priv->inactive_shadow.shadow_color[1] = 0;
     priv->inactive_shadow.shadow_color[2] = 0;
     priv->draggable_border_width  = DRAGGABLE_BORDER_WIDTH_DEFAULT;
-    priv->attach_modal_dialogs = ATTACH_MODAL_DIALOGS_DEFAULT;
     priv->blur_type = BLUR_TYPE_DEFAULT;
     priv->metacity_theme = g_strdup (METACITY_THEME_DEFAULT);
     priv->metacity_active_opacity = METACITY_ACTIVE_OPACITY_DEFAULT;
