@@ -104,7 +104,14 @@ moveInitiate (CompAction         *action,
 	ms->snapOffY  = y - workArea.y ();
 
 	if (!ms->grab)
-	    ms->grab = s->pushGrab (ms->moveCursor, "move");
+	{
+	    Cursor moveCursor = screen->cursorCache (XC_fleur);
+
+	    if (state & CompAction::StateInitButton)
+		ms->grab = s->pushPointerGrab (moveCursor, "move");
+	    else
+		ms->grab = s->pushGrab (moveCursor, "move");
+	}
 
 	if (ms->grab)
 	{
@@ -572,10 +579,10 @@ MoveScreen::handleEvent (XEvent *event)
 	case ClientMessage:
 	    if (event->xclient.message_type == Atoms::wmMoveResize)
 	    {
-		unsigned   long type = (unsigned long) event->xclient.data.l[2];
-
 		MOVE_SCREEN (screen);
 
+		unsigned long type = event->xclient.data.l[2];
+
 		if (type == WmMoveResizeMove ||
 		    type == WmMoveResizeMoveKeyboard)
 		{
@@ -711,7 +718,6 @@ MoveScreen::MoveScreen (CompScreen *screen) :
 	key[i] = XKeysymToKeycode (screen->dpy (),
 				   XStringToKeysym (mKeys[i].name));
 
-    moveCursor = XCreateFontCursor (screen->dpy (), XC_fleur);
     if (cScreen)
     {
 	CompositeScreenInterface::setHandler (cScreen);
@@ -734,9 +740,6 @@ MoveScreen::~MoveScreen ()
 {
     if (region)
 	XDestroyRegion (region);
-
-    if (moveCursor)
-	XFreeCursor (screen->dpy (), moveCursor);
 }
 
 bool
