@@ -48,7 +48,7 @@ ExpoScreen::dndInit (CompAction         *action,
 		     CompAction::State  state,
 		     CompOption::Vector &options)
 {
-    if (expoMode)
+    if (expoMode && expoCam == 1.0f)
     {
 	dndState = DnDStart;
 	action->setState (action->state () | CompAction::StateTermButton);
@@ -65,7 +65,7 @@ ExpoScreen::dndFini (CompAction         *action,
 		     CompAction::State  state,
 		     CompOption::Vector &options)
 {
-    if (dndState == DnDDuring || dndState == DnDStart)
+    if (expoCam == 1.0f && (dndState == DnDDuring || dndState == DnDStart))
     {
 	if (dndWindow)
 	    finishWindowMovement ();
@@ -141,10 +141,13 @@ ExpoScreen::termExpo (CompAction         *action,
     if (dndState != DnDNone)
 	dndFini (action, state, options);
 
-    if (state & CompAction::StateCancel)
-	vpUpdateMode = VPUpdatePrevious;
-    else
-	vpUpdateMode = VPUpdateMouseOver;
+    if (expoCam == 1.0f)
+    {
+	if (state & CompAction::StateCancel)
+	    vpUpdateMode = VPUpdatePrevious;
+	else
+	    vpUpdateMode = VPUpdateMouseOver;
+    }
 
     dndState  = DnDNone;
     dndWindow = NULL;
@@ -324,6 +327,7 @@ ExpoScreen::handleEvent (XEvent *event)
 
 	case ButtonPress:
 	    if (expoMode			    &&
+		expoCam == 1.0f			    &&
 		event->xbutton.button == Button1    &&
 		event->xbutton.root   == screen->root ())
 	    {
@@ -354,6 +358,7 @@ ExpoScreen::handleEvent (XEvent *event)
 
 	case ButtonRelease:
 	    if (expoMode			    &&
+		expoCam == 1.0f			    &&
 		event->xbutton.button == Button1    &&
 		event->xbutton.root   == screen->root ())
 	    {
@@ -778,10 +783,10 @@ ExpoScreen::paintWall (const GLScreenPaintAttrib &attrib,
 
     /* Zoom animation stuff */
     /* camera position for the selected viewport */
-    GLVector vpCamPos (0, 0, 0, 0);
+    GLVector vpCamPos (0, 0, 0, 1);
 
     /* camera position during expo mode */
-    GLVector expoCamPos (0, 0, 0, 0);
+    GLVector expoCamPos (0, 0, 0, 1);
 
     float sx = screen->width ()  / static_cast <float> (output->width ());
     float sy = screen->height () / static_cast <float> (output->height ());
@@ -1461,6 +1466,8 @@ ExpoWindow::glPaint (const GLWindowPaintAttrib &attrib,
 		     const CompRegion          &region,
 		     unsigned int              mask)
 {
+    GLMatrix            wTransform (transform);
+
     if (eScreen->expoActive)
     {
 	if (eScreen->expoCam > 0.0)
@@ -1486,9 +1493,26 @@ ExpoWindow::glPaint (const GLWindowPaintAttrib &attrib,
 
 	if (opacity <= 0)
 	    mask |= PAINT_WINDOW_NO_CORE_INSTANCE_MASK;
+
+	/* Stretch maximized windows a little so that you don't
+	 * have an awkward gap */
+	if (((window->state () & MAXIMIZE_STATE) == MAXIMIZE_STATE) &&
+	    (eScreen->dndWindow != window))
+	{
+	    CompOutput *o = &screen->outputDevs ()[screen->outputDeviceForGeometry(window->geometry())];
+	    float yS = 1.0 + ((o->height () / (float) window->height ()) - 1.0f) * sigmoidProgress (eScreen->expoCam);
+	    float xS = 1.0 + ((o->width () / (float) window->width ()) - 1.0f) * sigmoidProgress (eScreen->expoCam);
+	    wTransform.translate (window->x () + window->width (),
+				  window->y () + window->height (),
+				  0.0f);
+	    wTransform.scale (xS, yS, 1.0f);
+	    wTransform.translate (-(window->x () + window->width ()),
+				  -(window->y () + window->height ()),
+				  0.0f);
+	}
     }
 
-    return gWindow->glPaint (attrib, transform, region, mask);
+    return gWindow->glPaint (attrib, wTransform, region, mask);
 }
 
 bool
