@@ -27,6 +27,7 @@
 
 #include <core/core.h>
 #include <core/atoms.h>
+#include <X11/cursorfont.h>
 
 #include "resize-logic.h"
 
@@ -48,6 +49,16 @@ static const unsigned short TOUCH_RIGHT = 2;
 static const unsigned short TOUCH_TOP = 3;
 static const unsigned short TOUCH_BOTTOM = 4;
 
+#define LEFT_CURSOR XC_left_side
+#define RIGHT_CURSOR XC_right_side
+#define UP_CURSOR XC_top_side
+#define UP_LEFT_CURSOR XC_top_left_corner
+#define UP_RIGHT_CURSOR XC_top_right_corner
+#define DOWN_CURSOR XC_bottom_side
+#define DOWN_LEFT_CURSOR XC_bottom_left_corner
+#define DOWN_RIGHT_CURSOR XC_bottom_right_corner
+#define MIDDLE_CURSOR XC_fleur
+
 using namespace resize;
 
 ResizeLogic::ResizeLogic() :
@@ -65,7 +76,9 @@ ResizeLogic::ResizeLogic() :
     offWorkAreaConstrained (true),
     options (NULL),
     cScreen (NULL),
-    gScreen (NULL)
+    gScreen (NULL),
+    lastMaskX (0),
+    lastMaskY (0)
 {
     rKeys[0].name	= "Left";
     rKeys[0].dx		= -1;
@@ -203,6 +216,8 @@ ResizeLogic::handleEvent (XEvent *event)
 				    ((int) (event->xclient.data.l[3] ?
 				     event->xclient.data.l[3] : -1));
 
+				lastMaskX = lastMaskY = 0;
+
 				initiateResizeDefaultMode (
 				    &options->optionGetInitiateButton (),
 				    CompAction::StateInitButton, o);
@@ -411,7 +426,20 @@ ResizeLogic::handleKeyEvent (KeyCode keycode)
 
 		mask = rKeys[i].resizeMask;
 
-		mScreen->updateGrab (grabIndex, cursor[i]);
+		if ((mask & ResizeLeftMask || mask & ResizeRightMask) && (mask != lastMaskX))
+		{
+		    pointerDx *= -1;
+
+		    lastMaskX = mask;
+		}
+		if ((mask & ResizeUpMask || mask & ResizeDownMask) && (mask != lastMaskY))
+		{
+		    pointerDy *= -1;
+
+		    lastMaskY = mask;
+		}
+
+		mScreen->updateGrab (grabIndex, screen->cursorCache (keyCursorNames[i]));
 	    }
 	    break;
 	}
@@ -439,14 +467,14 @@ ResizeLogic::handleMotionEvent (int xRoot, int yRoot)
 	    accumulatePointerMotion (xRoot, yRoot);
 	}
 
-	if (mask & ResizeLeftMask)
+	if (mask & ResizeLeftMask || lastMaskX & ResizeLeftMask)
 	    wi -= pointerDx;
-	else if (mask & ResizeRightMask)
+	else if (mask & ResizeRightMask || lastMaskX & ResizeRightMask)
 	    wi += pointerDx;
 
-	if (mask & ResizeUpMask)
+	if (mask & ResizeUpMask || lastMaskY & ResizeUpMask)
 	    he -= pointerDy;
-	else if (mask & ResizeDownMask)
+	else if (mask & ResizeDownMask || lastMaskY & ResizeDownMask)
 	    he += pointerDy;
 
 	if (w->state () & CompWindowStateMaximizedVertMask)
@@ -635,36 +663,36 @@ ResizeLogic::getStretchRectangle (BoxPtr pBox)
 Cursor
 ResizeLogic::cursorFromResizeMask (unsigned int mask)
 {
-    Cursor cursor;
+    unsigned int cursor_name;
 
     if (mask & ResizeLeftMask)
     {
 	if (mask & ResizeDownMask)
-	    cursor = downLeftCursor;
+	    cursor_name = DOWN_LEFT_CURSOR;
 	else if (mask & ResizeUpMask)
-	    cursor = upLeftCursor;
+	    cursor_name = UP_LEFT_CURSOR;
 	else
-	    cursor = leftCursor;
+	    cursor_name = LEFT_CURSOR;
     }
     else if (mask & ResizeRightMask)
     {
 	if (mask & ResizeDownMask)
-	    cursor = downRightCursor;
+	    cursor_name = DOWN_RIGHT_CURSOR;
 	else if (mask & ResizeUpMask)
-	    cursor = upRightCursor;
+	    cursor_name = UP_RIGHT_CURSOR;
 	else
-	    cursor = rightCursor;
+	    cursor_name = RIGHT_CURSOR;
     }
     else if (mask & ResizeUpMask)
     {
-	cursor = upCursor;
+	cursor_name = UP_CURSOR;
     }
     else
     {
-	cursor = downCursor;
+	cursor_name = DOWN_CURSOR;
     }
 
-    return cursor;
+    return screen->cursorCache (cursor_name);
 }
 
 void
@@ -1310,7 +1338,7 @@ ResizeLogic::initiateResize (CompAction		*action,
 	    Cursor cursor;
 
 	    if (state & CompAction::StateInitKey)
-		cursor = middleCursor;
+		cursor = screen->cursorCache (MIDDLE_CURSOR);
 	    else
 		cursor = cursorFromResizeMask (mask);
 
