@@ -30,7 +30,7 @@
 #include <sys/time.h>
 
 #include <X11/Xatom.h>
-#include <X11/cursorfont.h>
+#include <X11/extensions/shape.h>
 
 #include <core/atoms.h>
 #include <scale/scale.h>
@@ -244,6 +244,70 @@ ScaleWindow::scalePaintDecoration (const GLWindowPaintAttrib& attrib,
 	    }
 	}
     }
+
+    if (spScreen->optionGetDndTimeoutSpinner () &&
+	priv->window->id() == spScreen->selectedWindow &&
+	priv->slot &&
+	spScreen->hover.active ())
+    {
+	GLTexture* spinner = NULL;
+
+	if (!spScreen->dndSpinners.empty ())
+	{
+	    int speed = spScreen->optionGetDndTimeoutSpinnerSpeed ();
+	    unsigned dndSpinnerIdx = (spScreen->hover.minTime() - spScreen->hover.minLeft()) / speed;
+	    const GLTexture::List& tex_list = spScreen->dndSpinners[dndSpinnerIdx % spScreen->dndSpinners.size()];
+	    spinner = tex_list.empty() ? NULL : tex_list.front();
+	}
+
+	if (spinner)
+	{
+	    float  scale;
+	    int    x, y;
+	    int    size;
+	    int    scaledWinWidth, scaledWinHeight;
+	    int    spinnerMaxSize = spinner->width();
+
+	    scaledWinWidth  = priv->window->width () * priv->scale;
+	    scaledWinHeight = priv->window->height () * priv->scale;
+
+	    size = MIN (spinnerMaxSize, MIN (scaledWinWidth, scaledWinHeight));
+	    x = priv->tx + priv->window->x () + (scaledWinWidth - size) / 2;
+	    y = priv->ty + priv->window->y () + (scaledWinHeight - size) / 2;
+	    scale = 1.0;
+
+	    if (size != spinnerMaxSize)
+	    {
+		size = spinnerMaxSize;
+		scale = MIN (scaledWinWidth, scaledWinHeight) / (float) spinnerMaxSize;
+	    }
+
+	    priv->cWindow->addDamageRect (CompRect (x, y, size / scale, size / scale));
+
+	    mask |= PAINT_WINDOW_BLEND_MASK;
+
+	    CompRegion            iconReg (0, 0, size, size);
+	    GLTexture::MatrixList ml (1);
+
+	    ml[0] = spinner->matrix ();
+	    priv->gWindow->vertexBuffer ()->begin ();
+
+	    if (size)
+		priv->gWindow->glAddGeometry (ml, iconReg, iconReg);
+
+	    if (priv->gWindow->vertexBuffer ()->end ())
+	    {
+		GLMatrix            wTransform (transform);
+		GLWindowPaintAttrib sAttrib (attrib);
+
+		wTransform.scale (scale, scale, 1.0f);
+		wTransform.translate (x / scale, y / scale, 0.0f);
+		sAttrib.brightness *= 0.65f;
+
+		priv->gWindow->glDrawTexture (spinner, wTransform, sAttrib, mask);
+	    }
+	}
+    }
 }
 
 bool
@@ -924,6 +988,7 @@ PrivateScaleScreen::donePaint ()
 		activateEvent (false);
 		state = ScaleScreen::Idle;
 
+		screen->handleEventSetEnabled (this, false);
 		cScreen->preparePaintSetEnabled (this, false);
 		cScreen->donePaintSetEnabled (this, false);
 		gScreen->glPaintOutputSetEnabled (this, false);
@@ -982,7 +1047,7 @@ PrivateScaleScreen::checkForWindowAt (int x, int y)
 }
 
 void
-PrivateScaleScreen::sendDndStatusMessage (Window source)
+PrivateScaleScreen::sendDndStatusMessage (Window source, bool asks)
 {
     XEvent xev;
 
@@ -994,11 +1059,36 @@ PrivateScaleScreen::sendDndStatusMessage (Window source)
     xev.xclient.window	     = source;
 
     xev.xclient.data.l[0] = dndTarget;
-    xev.xclient.data.l[1] = 2;
+    xev.xclient.data.l[1] = 0;
     xev.xclient.data.l[2] = 0;
     xev.xclient.data.l[3] = 0;
     xev.xclient.data.l[4] = None;
 
+    if (asks)
+    {
+	xev.xclient.data.l[1] = 1 << 0 | 1 << 1;
+	xev.xclient.data.l[4] = xdndActionAsk;
+    }
+
+    XSendEvent (screen->dpy (), source, false, 0, &xev);
+}
+
+void
+PrivateScaleScreen::sendDndFinishedMessage (Window source)
+{
+    XEvent xev;
+
+    xev.xclient.type    = ClientMessage;
+    xev.xclient.display = screen->dpy ();
+    xev.xclient.format  = 32;
+
+    xev.xclient.message_type = xdndFinished;
+    xev.xclient.window	     = source;
+
+    xev.xclient.data.l[0] = dndTarget;
+    xev.xclient.data.l[1] = 0; // Not accepted
+    xev.xclient.data.l[2] = None;
+
     XSendEvent (screen->dpy (), source, false, 0, &xev);
 }
 
@@ -1037,7 +1127,10 @@ PrivateScaleScreen::scaleTerminate (CompAction         *action,
     }
 
     if (ss->priv->dndTarget)
+    {
+	ss->priv->dndCheck.stop ();
 	XUnmapWindow (::screen->dpy (), ss->priv->dndTarget);
+    }
 
     ss->priv->grab = false;
 
@@ -1103,9 +1196,9 @@ PrivateScaleScreen::ensureDndRedirectWindow ()
     if (!dndTarget)
     {
 	XSetWindowAttributes attr;
-	long		     xdndVersion = 3;
+	long		     xdndVersion = 5;
 
-	attr.override_redirect = true;
+	attr.override_redirect = True;
 
 	dndTarget = XCreateWindow (screen->dpy (), screen->root (),
 				   0, 0, 1, 1, 0, CopyFromParent,
@@ -1118,9 +1211,20 @@ PrivateScaleScreen::ensureDndRedirectWindow ()
 			 (unsigned char *) &xdndVersion, 1);
     }
 
+    if (screen->XShape ())
+    {
+	CompRegion workAreaRegion;
+
+	foreach (const CompOutput& output, screen->outputDevs ())
+	    workAreaRegion |= output.workArea ();
+
+	XShapeCombineRegion (screen->dpy (), dndTarget, ShapeBounding, 0, 0, workAreaRegion.handle (), ShapeSet);
+    }
+
     XMoveResizeWindow (screen->dpy (), dndTarget,
 		       0, 0, screen->width (), screen->height ());
     XMapRaised (screen->dpy (), dndTarget);
+    XSync (screen->dpy (), False);
 
     return true;
 }
@@ -1212,7 +1316,7 @@ PrivateScaleScreen::scaleInitiateCommon (CompAction         *action,
 	}
 	else if (!grabIndex)
 	{
-	    grabIndex = screen->pushGrab (cursor, "scale");
+	    grabIndex = screen->pushGrab (screen->normalCursor (), "scale");
 	    if (grabIndex)
 		grab = true;
 	}
@@ -1249,6 +1353,7 @@ PrivateScaleScreen::scaleInitiateCommon (CompAction         *action,
 
 	cScreen->damageScreen ();
 
+	screen->handleEventSetEnabled (this, true);
 	cScreen->preparePaintSetEnabled (this, true);
 	cScreen->donePaintSetEnabled (this, true);
 	gScreen->glPaintOutputSetEnabled (this, true);
@@ -1531,35 +1636,34 @@ PrivateScaleScreen::windowRemove (CompWindow *w)
 	    }
 	    else
 	    {
-		CompOption::Vector o (0);
-		CompAction         *action;
-
 		/* terminate scale mode if the recently closed
 		 * window was the last scaled window */
-
-		o.push_back (CompOption ("root", CompOption::TypeInt));
-		o[0].value ().set ((int) screen->root ());
-
-		action = &optionGetInitiateEdge ();
-		scaleTerminate (action, CompAction::StateCancel, o);
-
-		action = &optionGetInitiateKey ();
-		scaleTerminate (action, CompAction::StateCancel, o);
+		terminateScale (false);
 		break;
 	    }
 	}
     }
 }
 
+void PrivateScaleScreen::terminateScale (bool accept)
+{
+    CompOption::Vector o (0);
+
+    o.push_back (CompOption ("root", CompOption::TypeInt));
+    o[0].value ().set ((int) screen->root ());
+
+    scaleTerminate (&optionGetInitiateEdge (), accept ? 0 : CompAction::StateCancel, o);
+    scaleTerminate (&optionGetInitiateKey (), accept ? 0 : CompAction::StateCancel, o);
+
+    activateEvent (false);
+}
+
 bool
 PrivateScaleScreen::hoverTimeout ()
 {
     if (grab && state != ScaleScreen::In)
     {
-	CompWindow         *w;
-	CompOption::Vector o (0);
-
-	w = screen->findWindow (selectedWindow);
+	CompWindow *w = screen->findWindow (selectedWindow);
 	if (w)
 	{
 	    lastActiveNum    = w->activeNum ();
@@ -1568,16 +1672,42 @@ PrivateScaleScreen::hoverTimeout ()
 	    w->moveInputFocusTo ();
 	}
 
-	o.push_back (CompOption ("root", CompOption::TypeInt));
-	o[0].value ().set ((int) screen->root ());
-
-	scaleTerminate (&optionGetInitiateEdge (), 0, o);
-	scaleTerminate (&optionGetInitiateKey (), 0, o);
+	terminateScale (true);
     }
 
     return false;
 }
 
+bool
+PrivateScaleScreen::dndCheckTimeout ()
+{
+    if (!dndTarget)
+	return false;
+
+    CompWindow *w = screen->findWindow (dndTarget);
+
+    if (!w || !w->isMapped ())
+	return false;
+
+    Window drag_owner = XGetSelectionOwner (screen->dpy (), xdndSelection);
+
+    if (drag_owner)
+    {
+	// evil hack because some apps (Qt) don't release the selection owner on drag finished
+	Window root_r, child_r;
+	int root_x_r, root_y_r, win_x_r, win_y_r;
+	unsigned int mask;
+	XQueryPointer (screen->dpy (), screen->root (), &root_r, &child_r,
+		       &root_x_r, &root_y_r, &win_x_r, &win_y_r, &mask);
+
+	if (mask & (Button1Mask | Button2Mask | Button3Mask))
+	    return true;
+    }
+
+    terminateScale (false);
+    return false;
+}
+
 void
 PrivateScaleScreen::handleEvent (XEvent *event)
 {
@@ -1606,21 +1736,17 @@ PrivateScaleScreen::handleEvent (XEvent *event)
 		state != ScaleScreen::In)
 	    {
 		XButtonEvent       *button = &event->xbutton;
-		CompOption::Vector o (0);
-
-		o.push_back (CompOption ("root", CompOption::TypeInt));
-		o[0].value ().set ((int) screen->root ());
 
 		/* Button1 terminates scale mode, other buttons can select
 		 * windows */
-		if (selectWindowAt (button->x_root, button->y_root, true) &&
-		    event->xbutton.button == Button1)
+		if (event->xbutton.button != Button1)
+		    break;
+
+		if (selectWindowAt (button->x_root, button->y_root, true))
 		{
-		    scaleTerminate (&optionGetInitiateEdge (), 0, o);
-		    scaleTerminate (&optionGetInitiateKey (), 0, o);
+		    terminateScale (true);
 		}
-		else if (optionGetClickOnDesktop () == 1 &&
-			 event->xbutton.button == Button1)
+		else if (optionGetClickOnDesktop () != ScaleOptions::ClickOnDesktopNone)
 		{
 		    CompPoint pointer (button->x_root, button->y_root);
 		    CompRect  workArea (screen->workArea ());
@@ -1629,23 +1755,10 @@ PrivateScaleScreen::handleEvent (XEvent *event)
 
 		    if (workArea.contains (pointer))
 		    {
-			scaleTerminate (&optionGetInitiateEdge (), 0, o);
-			scaleTerminate (&optionGetInitiateKey (), 0, o);
-			screen->enterShowDesktopMode ();
-		    }
-		}
-		else if (optionGetClickOnDesktop () == 2 &&
-			 event->xbutton.button == Button1)
-		{
-		    CompPoint pointer (button->x_root, button->y_root);
-		    CompRect  workArea (screen->workArea ());
-		    workArea.setX (workArea.x() + optionGetXOffset ());
-		    workArea.setY (workArea.y() + optionGetYOffset ());
+			terminateScale (false);
 
-		    if (workArea.contains (pointer))
-		    {
-			scaleTerminate (&optionGetInitiateEdge (), 0, o);
-			scaleTerminate (&optionGetInitiateKey (), 0, o);
+			if (optionGetClickOnDesktop () == ScaleOptions::ClickOnDesktopShowDesktop)
+			    screen->enterShowDesktopMode ();
 		    }
 		}
 	    }
@@ -1669,33 +1782,31 @@ PrivateScaleScreen::handleEvent (XEvent *event)
 	    w = screen->findWindow (event->xdestroywindow.window);
 	    break;
 	case UnmapNotify:
-
 	     w = screen->findWindow (event->xunmap.window);
 	     break;
 	case ClientMessage:
 	    if (event->xclient.message_type == Atoms::xdndPosition)
 	    {
-		w = screen->findWindow (event->xclient.window);
-		if (w)
+		if (event->xclient.window == dndTarget)
 		{
-		    if (w->id () == dndTarget)
-			sendDndStatusMessage (event->xclient.data.l[0]);
+		    bool acceptsDnd = false;
 
-		    if (grab			 &&
-			state != ScaleScreen::In &&
-			w->id () == dndTarget)
+		    if (grab && state != ScaleScreen::In)
 		    {
+			dndCheck.stop ();
+
 			ScaleWindow *sw = checkForWindowAt (pointerX, pointerY);
 			if (sw && sw->priv->isScaleWin ())
 			{
 			    int time;
 
 			    time = optionGetHoverTime ();
+			    acceptsDnd = true;
 
 			    if (hover.active ())
 			    {
 				int lastMotion = sqrt (pow (pointerX - lastPointerX, 2) + pow (pointerY - lastPointerY, 2));
-				
+
 				if (sw->window->id () != selectedWindow || lastMotion > optionGetDndDistance ())
 				    hover.stop ();
 			    }
@@ -1713,24 +1824,35 @@ PrivateScaleScreen::handleEvent (XEvent *event)
 				hover.stop ();
 			}
 		    }
+
+		    sendDndStatusMessage (event->xclient.data.l[0], acceptsDnd);
 		}
 	    }
-	    else if (event->xclient.message_type == Atoms::xdndDrop ||
-		     event->xclient.message_type == Atoms::xdndLeave)
+	    else if (event->xclient.message_type == Atoms::xdndEnter)
 	    {
-		w = screen->findWindow (event->xclient.window);
-		if (w)
+		if (event->xclient.window == dndTarget &&
+		    grab && state != ScaleScreen::In)
 		{
-		    if (grab			 &&
-			state != ScaleScreen::In &&
-			w->id () == dndTarget)
-		    {
-			CompOption::Vector o (0);
-			o.push_back (CompOption ("root", CompOption::TypeInt));
-			o[0].value ().set ((int) screen->root ());
+		    dndCheck.stop ();
+		}
+	    }
+	    else if (event->xclient.message_type == Atoms::xdndLeave)
+	    {
+		if (event->xclient.window == dndTarget &&
+		    grab && state != ScaleScreen::In)
+		{
+		    dndCheck.start ();
+		}
+	    }
+	    else if (event->xclient.message_type == Atoms::xdndDrop)
+	    {
+		if (event->xclient.window == dndTarget)
+		{
+		    sendDndFinishedMessage (event->xclient.data.l[0]);
 
-			scaleTerminate (&optionGetInitiateEdge (), 0, o);
-			scaleTerminate (&optionGetInitiateKey (), 0, o);
+		    if (grab && state != ScaleScreen::In)
+		    {
+			terminateScale (true);
 		    }
 		}
 	    }
@@ -1746,6 +1868,7 @@ PrivateScaleScreen::handleEvent (XEvent *event)
 
     switch (event->type) {
 	case UnmapNotify:
+	    dndCheck.start ();
 	    if (w)
 		windowRemove (w);
 	    break;
@@ -1832,9 +1955,11 @@ PrivateScaleScreen::PrivateScaleScreen (CompScreen *s) :
     grab (false),
     grabIndex (0),
     dndTarget (None),
+    xdndSelection (XInternAtom (screen->dpy (), "XdndSelection", False)),
+    xdndFinished (XInternAtom (screen->dpy (), "XdndFinished", False)),
+    xdndActionAsk (XInternAtom (screen->dpy (), "XdndActionAsk", False)),
     state (ScaleScreen::Idle),
     moreAdjust (false),
-    cursor (0),
     nSlots (0)
 {
     leftKeyCode  = XKeysymToKeycode (screen->dpy (), XStringToKeysym ("Left"));
@@ -1842,11 +1967,11 @@ PrivateScaleScreen::PrivateScaleScreen (CompScreen *s) :
     upKeyCode    = XKeysymToKeycode (screen->dpy (), XStringToKeysym ("Up"));
     downKeyCode  = XKeysymToKeycode (screen->dpy (), XStringToKeysym ("Down"));
 
-    cursor = XCreateFontCursor (screen->dpy (), XC_left_ptr);
-
     opacity = (OPAQUE * optionGetOpacity ()) / 100;
 
     hover.setCallback (boost::bind (&PrivateScaleScreen::hoverTimeout, this));
+    dndCheck.setCallback (boost::bind (&PrivateScaleScreen::dndCheckTimeout, this));
+    dndCheck.setTimes (200);
 
     optionSetOpacityNotify (boost::bind (&PrivateScaleScreen::updateOpacity, this));
 
@@ -1883,15 +2008,35 @@ PrivateScaleScreen::PrivateScaleScreen (CompScreen *s) :
 
 #undef SCALEBIND
 
+    CompString pluginName("scale");
+    CompSize size;
+    CompString file;
+
+    file = "dnd-spinner-000.png";
+    dndSpinners.push_back(GLTexture::readImageToTexture(file, pluginName, size));
+    file = "dnd-spinner-125.png";
+    dndSpinners.push_back(GLTexture::readImageToTexture(file, pluginName, size));
+    file = "dnd-spinner-250.png";
+    dndSpinners.push_back(GLTexture::readImageToTexture(file, pluginName, size));
+    file = "dnd-spinner-375.png";
+    dndSpinners.push_back(GLTexture::readImageToTexture(file, pluginName, size));
+    file = "dnd-spinner-500.png";
+    dndSpinners.push_back(GLTexture::readImageToTexture(file, pluginName, size));
+    file = "dnd-spinner-625.png";
+    dndSpinners.push_back(GLTexture::readImageToTexture(file, pluginName, size));
+    file = "dnd-spinner-750.png";
+    dndSpinners.push_back(GLTexture::readImageToTexture(file, pluginName, size));
+    file = "dnd-spinner-875.png";
+    dndSpinners.push_back(GLTexture::readImageToTexture(file, pluginName, size));
+
     ScreenInterface::setHandler (s);
     CompositeScreenInterface::setHandler (cScreen, false);
     GLScreenInterface::setHandler (gScreen, false);
-}
 
-PrivateScaleScreen::~PrivateScaleScreen ()
-{
-    if (cursor)
-	XFreeCursor (screen->dpy (), cursor);
+    screen->handleEventSetEnabled (this, false);
+    cScreen->preparePaintSetEnabled (this, false);
+    cScreen->donePaintSetEnabled (this, false);
+    gScreen->glPaintOutputSetEnabled (this, false);
 }
 
 void
