@@ -239,6 +239,20 @@ class PluginManager
 	CompStringSet blacklist;
 };
 
+enum GrabType {
+    POINTER = 1 << 0,
+    KEYBOARD = 1 << 1,
+    ALL = POINTER|KEYBOARD
+};
+
+struct Grab {
+    Grab (GrabType type, Cursor cursor, const char *name) : type(type), cursor(cursor), name(name) {}
+    Grab (Cursor cursor, const char *name) : Grab(GrabType::ALL, cursor, name) {}
+    GrabType   type;
+    Cursor     cursor;
+    const char *name;
+};
+
 class GrabList
 {
     // TODO: std::list<Grab *> is almost certainly the wrong data
@@ -249,13 +263,14 @@ class GrabList
 public:
     typedef GrabPtrList::iterator GrabIterator;
 
-    bool grabsEmpty() const { return grabs.empty(); }
-    void grabsPush(Grab* grab) { grabs.push_back (grab); }
-    GrabIterator grabsBegin() { return grabs.begin(); }
-    GrabIterator grabsEnd() { return grabs.end(); }
-    void grabsRemove(Grab* grab);
+    bool grabsEmpty () const { return grabs.empty (); }
+    void grabsPush (Grab* grab) { grabs.push_back (grab); }
+    GrabIterator grabsBegin () { return begin (grabs); }
+    GrabIterator grabsEnd () { return end (grabs); }
+    void grabsRemove (Grab* grab);
     bool grabExist (const char *grab);
-    Grab* grabsBack() { return grabs.back (); }
+    Grab* topGrab (GrabType t) { auto it = std::find_if (begin (grabs), end (grabs), [t] (Grab *g) { return (g->type & t); }); return (it != end (grabs)) ? *it : NULL; }
+    Grab* grabsBack () { return grabs.back (); }
 
 private:
     GrabPtrList grabs;
@@ -349,12 +364,6 @@ class ButtonGrab {
 	int          count;
 };
 
-struct Grab {
-	Grab(Cursor cursor, const char *name) : cursor(cursor), name(name) {}
-	Cursor     cursor;
-	const char *name;
-};
-
 // data members that don't belong (these probably belong
 // in CompScreenImpl as PrivateScreen doesn't use them)
 struct OrphanData : boost::noncopyable
@@ -571,9 +580,6 @@ private:
     Display* const& dpy;
 };
 
-
-
-
 unsigned int windowStateMask (Atom state);
 
 }} // namespace compiz::private_screen
@@ -680,6 +686,8 @@ class PrivateScreen :
 
 	void updateScreenInfo ();
 
+	void updateResources ();
+
 	Window getActiveWindow (Window root);
 
 	void setWindowState (unsigned int state, Window id);
@@ -700,9 +708,17 @@ class PrivateScreen :
 
 	void setDefaultWindowAttributes (XWindowAttributes *);
 
+	void updateCursors (const CompString& theme, int size);
+
+	Cursor cursorCache (unsigned int cursorName);
+
 	static void compScreenSnEvent (SnMonitorEvent *event,
 			   void           *userData);
 
+	CompScreen::GrabHandle pushGrabGeneric (::compiz::private_screen::GrabType type,
+						Cursor cursor,
+						const char *name);
+
 	int  getXkbEvent() const { return xkbEvent.get(); }
 	std::vector<XineramaScreenInfo>& getScreenInfo () { return screenInfo; }
 	SnDisplay* getSnDisplay () const { return snDisplay; }
@@ -763,9 +779,8 @@ public:
     Window wmSnSelectionWindow;
 
     int    clientPointerDeviceId;
-    Cursor normalCursor;
-    Cursor busyCursor;
     Cursor invisibleCursor;
+    std::vector<Cursor> cursors;
     CompRect workArea;
 
     bool initialized;
@@ -802,6 +817,7 @@ private:
     Window xdndWindow;
     compiz::private_screen::PluginManager pluginManager;
     compiz::private_screen::WindowManager& windowManager;
+    CompOption::Vector resourceManager;
 };
 
 class CompManager
@@ -954,8 +970,8 @@ class CompScreenImpl : public CompScreen,
 	void unhookServerWindow (CompWindow *w);
 
 	Cursor normalCursor ();
-
 	Cursor invisibleCursor ();
+	Cursor cursorCache (unsigned int cursorName);
 
 	/* Adds an X Pointer and Keyboard grab to the stack. Since
 	 * compiz as a client only need to grab once, multiple clients
@@ -963,6 +979,12 @@ class CompScreenImpl : public CompScreen,
 	 * be grabbed once and the actual grab refcounted */
 	GrabHandle pushGrab (Cursor cursor, const char *name);
 
+	/* Adds an X Pointer grab to the stack. */
+	GrabHandle pushPointerGrab (Cursor cursor, const char *name);
+
+	/* Adds an X Keyboard grab to the stack. */
+    	GrabHandle pushKeyboardGrab (const char *name);
+
 	/* Allows you to change the pointer of your grab */
 	void updateGrab (GrabHandle handle, Cursor cursor);
 
@@ -1158,6 +1180,7 @@ class CompScreenImpl : public CompScreen,
         virtual void _matchExpHandlerChanged();
         virtual void _matchPropertyChanged(CompWindow *);
         virtual void _outputChangeNotify();
+        virtual void _cursorChangeNotify(const CompString&, int);
 
 	void grabServer ();
 	void ungrabServer ();
