@@ -400,6 +400,7 @@ class History : public virtual ::compiz::History,
 {
     public:
 	History();
+	virtual ~History () {}
 
 	void setCurrentActiveWindowHistory (int x, int y);
 
@@ -455,6 +456,7 @@ class StartupSequence : boost::noncopyable
 {
     public:
 	StartupSequence();
+	virtual ~StartupSequence() {}
 	void addSequence (SnStartupSequence *sequence, CompPoint const& vp);
 	void removeSequence (SnStartupSequence *sequence);
 	void removeAllSequences ();
@@ -516,6 +518,7 @@ public virtual ::compiz::Ping
 {
 public:
     Ping() : lastPing_(1) {}
+    virtual ~Ping() {}
     bool handlePingTimeout (WindowManager::iterator begin, WindowManager::iterator end, Display* dpy);
     unsigned int lastPing () const { return lastPing_; }
 
@@ -528,6 +531,7 @@ class DesktopWindowCount :
 {
 public:
     DesktopWindowCount();
+    virtual ~DesktopWindowCount() {}
     virtual void incrementDesktopWindowCount();
     virtual void decrementDesktopWindowCount();
     virtual int desktopWindowCount();
@@ -540,6 +544,7 @@ class MapNum :
 {
 public:
     MapNum();
+    virtual ~MapNum() {}
     virtual unsigned int nextMapNum();
 
 private:
@@ -552,6 +557,7 @@ class XWindowInfo :
 public:
     XWindowInfo(Display* const& dpy) :
 	dpy(dpy) {}
+    virtual ~XWindowInfo() {}
 
     virtual int getWmState (Window id);
     virtual void setWmState (int state, Window id) const;
@@ -618,15 +624,30 @@ class PrivateScreen :
 					   XButtonEvent       *event,
 					   CompOption::Vector &arguments);
 
+	bool shouldTriggerKeyPressAction (CompAction *action,
+					  XKeyEvent  *event);
+
+	bool shouldTriggerKeyReleaseAction (CompAction *action,
+					    XKeyEvent  *event);
+
+	bool shouldTriggerModifierPressAction (CompAction          *action,
+					       XkbStateNotifyEvent *event);
+
+	bool shouldTriggerModifierReleaseAction (CompAction          *action,
+						 XkbStateNotifyEvent *event);
+
 	bool triggerKeyPressBindings (CompOption::Vector &options,
+				      CompAction::Vector &actions,
 				      XKeyEvent          *event,
 				      CompOption::Vector &arguments);
 
 	bool triggerKeyReleaseBindings (CompOption::Vector &options,
+					CompAction::Vector &actions,
 					XKeyEvent          *event,
 					CompOption::Vector &arguments);
 
 	bool triggerStateNotifyBindings (CompOption::Vector  &options,
+					 CompAction::Vector  &actions,
 					 XkbStateNotifyEvent *event,
 					 CompOption::Vector  &arguments);
 
@@ -659,6 +680,8 @@ class PrivateScreen :
 
 	void updateScreenInfo ();
 
+	void updateResources ();
+
 	Window getActiveWindow (Window root);
 
 	void setWindowState (unsigned int state, Window id);
@@ -679,6 +702,10 @@ class PrivateScreen :
 
 	void setDefaultWindowAttributes (XWindowAttributes *);
 
+	void updateCursors (const CompString& theme, int size);
+
+	Cursor cursorCache (unsigned int cursorName);
+
 	static void compScreenSnEvent (SnMonitorEvent *event,
 			   void           *userData);
 
@@ -742,9 +769,8 @@ public:
     Window wmSnSelectionWindow;
 
     int    clientPointerDeviceId;
-    Cursor normalCursor;
-    Cursor busyCursor;
     Cursor invisibleCursor;
+    std::vector<Cursor> cursors;
     CompRect workArea;
 
     bool initialized;
@@ -781,6 +807,7 @@ private:
     Window xdndWindow;
     compiz::private_screen::PluginManager pluginManager;
     compiz::private_screen::WindowManager& windowManager;
+    CompOption::Vector resourceManager;
 };
 
 class CompManager
@@ -933,8 +960,8 @@ class CompScreenImpl : public CompScreen,
 	void unhookServerWindow (CompWindow *w);
 
 	Cursor normalCursor ();
-
 	Cursor invisibleCursor ();
+	Cursor cursorCache (unsigned int cursorName);
 
 	/* Adds an X Pointer and Keyboard grab to the stack. Since
 	 * compiz as a client only need to grab once, multiple clients
@@ -1137,6 +1164,7 @@ class CompScreenImpl : public CompScreen,
         virtual void _matchExpHandlerChanged();
         virtual void _matchPropertyChanged(CompWindow *);
         virtual void _outputChangeNotify();
+        virtual void _cursorChangeNotify(const CompString&, int);
 
 	void grabServer ();
 	void ungrabServer ();
