@@ -81,6 +81,7 @@ public:
     MOCK_METHOD0(_matchExpHandlerChanged, void ());
     MOCK_METHOD1(_matchPropertyChanged, void (CompWindow *));
     MOCK_METHOD0(_outputChangeNotify, void ());
+    MOCK_METHOD2(_cursorChangeNotify, void (const CompString&, int));
 
     MOCK_METHOD0(outputDevs, CompOutput::vector & ());
     MOCK_METHOD2(setWindowState, void (unsigned int state, Window id));
@@ -175,6 +176,7 @@ public:
     MOCK_METHOD0(displayString, const char * ());
     MOCK_METHOD0(getCurrentOutputExtents, CompRect ());
     MOCK_METHOD0(normalCursor, Cursor ());
+    MOCK_METHOD1(cursorCache, Cursor (unsigned int));
     MOCK_METHOD0(grabbed, bool ());
     MOCK_METHOD0(snDisplay, SnDisplay * ());
     MOCK_CONST_METHOD0(createFailed, bool ());
@@ -352,7 +354,7 @@ PluginFilesystem::PluginFilesystem()
 // tell GLib not to use the slice-allocator implementation
 // and avoid spurious valgrind reporting
 void glib_nice_for_valgrind() { setenv("G_SLICE", "always-malloc", true); }
-int const init = (glib_nice_for_valgrind(), 0);
+__attribute__((unused)) int const init = (glib_nice_for_valgrind(), 0);
 
 PluginFilesystem const* PluginFilesystem::instance = 0;
 
