@@ -37,38 +37,10 @@
 #include <X11/extensions/Xrender.h>
 #include <X11/Xregion.h>
 
-#ifdef HAVE_GTK_2_24
-
-#ifndef GDK_DISABLE_DEPRECATED
-#define GDK_DISABLE_DEPRECATED
-#endif
-
-#define create_foreign_window(xid)						       \
-    gdk_x11_window_foreign_new_for_display (gdk_display_get_default (),	       \
-					    xid)
-#else
-
-#define create_foreign_window(xid)						       \
-    gdk_window_foreign_new (xid)
-
-#ifdef GDK_DISABLE_DEPRECATED
-#undef GDK_DISABLE_DEPRECATED
-#endif
-
-#endif
-
-#ifndef GTK_DISABLE_DEPRECATED
-#define GTK_DISABLE_DEPRECATED
-#endif
-
 #include <gtk/gtk.h>
 #include <gdk/gdkx.h>
 #include <gdk/gdk.h>
 
-#ifdef USE_GCONF
-#include <gconf/gconf-client.h>
-#endif
-
 #ifdef USE_DBUS_GLIB
 #define DBUS_API_SUBJECT_TO_CHANGE
 #include <dbus/dbus.h>
@@ -77,11 +49,6 @@
 
 #define WNCK_I_KNOW_THIS_IS_UNSTABLE
 #include <libwnck/libwnck.h>
-#include <libwnck/window-action-menu.h>
-
-#ifndef HAVE_LIBWNCK_2_19_4
-#define wnck_window_get_client_window_geometry wnck_window_get_geometry
-#endif
 
 #include <cairo.h>
 #include <cairo-xlib.h>
@@ -111,108 +78,11 @@
 
 #include <gwd-fwd.h>
 
-#define METACITY_GCONF_DIR "/apps/metacity/general"
-#define MUTTER_GCONF_DIR "/apps/mutter/general"
-
-#define COMPIZ_USE_SYSTEM_FONT_KEY		    \
-METACITY_GCONF_DIR "/titlebar_uses_system_font"
-		    
-#define COMPIZ_TITLEBAR_FONT_KEY	\
-METACITY_GCONF_DIR "/titlebar_font"
-
-#define COMPIZ_DOUBLE_CLICK_TITLEBAR_KEY	       \
-METACITY_GCONF_DIR "/action_double_click_titlebar"
-
-#define COMPIZ_MIDDLE_CLICK_TITLEBAR_KEY	       \
-METACITY_GCONF_DIR "/action_middle_click_titlebar"
-
-#define COMPIZ_RIGHT_CLICK_TITLEBAR_KEY	       \
-METACITY_GCONF_DIR "/action_right_click_titlebar"
-
-#define MUTTER_DRAGGABLE_BORDER_WIDTH_KEY \
-MUTTER_GCONF_DIR "/draggable_border_width"
-
-#define MUTTER_ATTACH_MODAL_DIALOGS_KEY \
-MUTTER_GCONF_DIR "/attach_modal_dialogs"
-
-#define META_THEME_KEY		\
-METACITY_GCONF_DIR "/theme"
-
-#define META_BUTTON_LAYOUT_KEY		\
-METACITY_GCONF_DIR "/button_layout"
-
-#define GCONF_DIR "/apps/gwd"
-
-#define USE_META_THEME_KEY	    \
-GCONF_DIR "/use_metacity_theme"
-
-#define META_THEME_OPACITY_KEY	        \
-GCONF_DIR "/metacity_theme_opacity"
-
-#define META_THEME_SHADE_OPACITY_KEY	      \
-GCONF_DIR "/metacity_theme_shade_opacity"
-
-#define META_THEME_ACTIVE_OPACITY_KEY	       \
-GCONF_DIR "/metacity_theme_active_opacity"
-
-#define META_THEME_ACTIVE_SHADE_OPACITY_KEY          \
-GCONF_DIR "/metacity_theme_active_shade_opacity"
-
-#define BLUR_TYPE_KEY	   \
-GCONF_DIR "/blur_type"
-
-#define WHEEL_ACTION_KEY   \
-GCONF_DIR "/mouse_wheel_action"
-
-#define USE_TOOLTIPS_KEY \
-GCONF_DIR "/use_tooltips"
-
-#define DBUS_DEST       "org.freedesktop.compiz"
-#define DBUS_PATH       "/org/freedesktop/compiz/decor/screen0"
-#define DBUS_INTERFACE  "org.freedesktop.compiz"
-#define DBUS_METHOD_GET "get"
-
-extern const float STROKE_ALPHA;
-
 extern const unsigned short ICON_SPACE;
 
-extern const float DOUBLE_CLICK_DISTANCE;
-
-#define WM_MOVERESIZE_SIZE_TOPLEFT      0
-#define WM_MOVERESIZE_SIZE_TOP          1
-#define WM_MOVERESIZE_SIZE_TOPRIGHT     2
-#define WM_MOVERESIZE_SIZE_RIGHT        3
-#define WM_MOVERESIZE_SIZE_BOTTOMRIGHT  4
-#define WM_MOVERESIZE_SIZE_BOTTOM       5
-#define WM_MOVERESIZE_SIZE_BOTTOMLEFT   6
-#define WM_MOVERESIZE_SIZE_LEFT         7
-#define WM_MOVERESIZE_MOVE              8
-#define WM_MOVERESIZE_SIZE_KEYBOARD     9
-#define WM_MOVERESIZE_MOVE_KEYBOARD    10
-
-extern const float	    SHADOW_RADIUS;
-extern const float	    SHADOW_OPACITY;
-extern const unsigned short SHADOW_OFFSET_X;
-extern const unsigned short SHADOW_OFFSET_Y;
-#define SHADOW_COLOR_RED   0x0000
-#define SHADOW_COLOR_GREEN 0x0000
-#define SHADOW_COLOR_BLUE  0x0000
-
-extern const float META_OPACITY;
-#define META_SHADE_OPACITY	    TRUE;
-extern const float META_ACTIVE_OPACITY;
-#define META_ACTIVE_SHADE_OPACITY   TRUE;
-
 #define META_MAXIMIZED (WNCK_WINDOW_STATE_MAXIMIZED_HORIZONTALLY | \
 WNCK_WINDOW_STATE_MAXIMIZED_VERTICALLY)
 
-#define CMDLINE_OPACITY              (1 << 0)
-#define CMDLINE_OPACITY_SHADE        (1 << 1)
-#define CMDLINE_ACTIVE_OPACITY       (1 << 2)
-#define CMDLINE_ACTIVE_OPACITY_SHADE (1 << 3)
-#define CMDLINE_BLUR                 (1 << 4)
-#define CMDLINE_THEME                (1 << 5)
-
 #define MWM_HINTS_DECORATIONS (1L << 1)
 
 #define MWM_DECOR_ALL      (1L << 0)
@@ -233,16 +103,13 @@ unsigned long decorations;
 
 extern gboolean minimal;
 
-#define SWITCHER_SPACE 40
-
 extern GWDSettingsNotified *notified;
-extern GWDSettings	   *settings;
+extern GWDSettings *settings;
 extern GWDSettingsWritable *writable;
 
 extern gdouble decoration_alpha;
 #ifdef USE_METACITY
-extern MetaButtonLayout   meta_button_layout;
-extern gboolean	          meta_button_layout_set;
+extern gboolean meta_button_layout_set;
 #endif
 
 extern Atom frame_input_window_atom;
@@ -264,6 +131,8 @@ extern Atom net_wm_state_modal_atom;
 extern Atom decor_request_atom;
 extern Atom decor_pending_atom;
 extern Atom decor_delete_pixmap_atom;
+extern Atom utf8_string_atom;
+extern Atom gtk_theme_variant_atom;
 
 extern Time dm_sn_timestamp;
 
@@ -393,8 +262,8 @@ typedef struct _decor {
     Box		      *last_pos_entered;
     guint	      button_states[BUTTON_NUM];
     Pixmap            x11Pixmap;
-    GdkPixmap	      *pixmap;
-    GdkPixmap	      *buffer_pixmap;
+    cairo_surface_t   *surface;
+    cairo_surface_t   *buffer_surface;
     GdkWindow	      *frame_window;
     GtkWidget         *decor_window;
     GtkWidget	      *decor_event_box;
@@ -413,8 +282,9 @@ typedef struct _decor {
     gboolean	      active;
     PangoLayout	      *layout;
     gchar	      *name;
+    gchar	      *gtk_theme_variant;
     cairo_pattern_t   *icon;
-    GdkPixmap	      *icon_pixmap;
+    cairo_surface_t   *icon_surface;
     GdkPixbuf	      *icon_pixbuf;
     WnckWindowState   state;
     WnckWindowActions actions;
@@ -495,22 +365,11 @@ extern decor_t    *switcher_window;
 extern XRenderPictFormat *xformat_rgba;
 extern XRenderPictFormat *xformat_rgb;
 
-/* gtk-window-decorator.c */
-
-double
-dist (double x1, double y1,
-      double x2, double y2);
-
 /* frames.c */
 
 void
 initialize_decorations ();
 
-void
-update_frames_border_extents (gpointer key,
-			      gpointer value,
-			      gpointer user_data);
-
 decor_frame_t *
 gwd_get_decor_frame (const gchar *);
 
@@ -547,30 +406,6 @@ frame_update_shadow (decor_frame_t	    *frame,
 void
 frame_update_titlebar_font (decor_frame_t *frame);
 
-void
-bare_frame_update_shadow (Display		  *xdisplay,
-			   Screen		  *screen,
-			   decor_frame_t	  *frame,
-			   decor_shadow_t	  **shadow_normal,
-			   decor_context_t	  *context_normal,
-			   decor_shadow_t	  **shadow_max,
-			   decor_context_t	  *context_max,
-			   decor_shadow_info_t    *info,
-			   decor_shadow_options_t *opt_shadow,
-			   decor_shadow_options_t *opt_no_shadow);
-
-void
-decor_frame_update_shadow (Display		  *xdisplay,
-			   Screen		  *screen,
-			   decor_frame_t	  *frame,
-			   decor_shadow_t	  **shadow_normal,
-			   decor_context_t	  *context_normal,
-			   decor_shadow_t	  **shadow_max,
-			   decor_context_t	  *context_max,
-			   decor_shadow_info_t    *info,
-			   decor_shadow_options_t *opt_shadow,
-			   decor_shadow_options_t *opt_no_shadow);
-
 decor_frame_t *
 create_normal_frame (const gchar *type);
 
@@ -590,12 +425,6 @@ update_window_decoration_size (WnckWindow *win);
 gboolean
 request_update_window_decoration_size (WnckWindow *win);
 
-void
-update_window_decoration_name (WnckWindow *win);
-
-gint
-max_window_name_width (WnckWindow *win);
-
 unsigned int
 populate_frame_type (decor_t *d);
 
@@ -626,9 +455,6 @@ update_shadow (void);
 void
 update_titlebar_font ();
 
-void
-update_window_decoration_name (WnckWindow *win);
-
 void
 update_window_decoration (WnckWindow *win);
 
@@ -649,10 +475,6 @@ decorations_changed (WnckScreen *screen);
 void
 connect_screen (WnckScreen *screen);
 
-void
-window_opened (WnckScreen *screen,
-	       WnckWindow *window);
-
 void
 window_closed (WnckScreen *screen,
 	       WnckWindow *window);
@@ -669,10 +491,6 @@ void
 restack_window (WnckWindow *win,
 		int	   stack_mode);
 
-void
-connect_window (WnckWindow *win);
-
-
 /* blur.c */
 
 void
@@ -780,34 +598,27 @@ cairo_get_shadow (decor_frame_t *, decor_shadow_options_t *opts, gboolean active
 
 /* gdk.c */
 
-void
-gdk_cairo_set_source_color_alpha (cairo_t  *cr,
-				  GdkColor *color,
-				  double   alpha);
-
 GdkWindow *
 create_gdk_window (Window xframe);
 
-GdkColormap *
-get_colormap_for_drawable (GdkDrawable *d);
-
 XRenderPictFormat *
-get_format_for_drawable (decor_t *d, GdkDrawable *drawable);
+get_format_for_surface (decor_t *d, cairo_surface_t *surface);
 
-GdkPixmap *
-create_pixmap (int	 w,
+cairo_surface_t *
+create_surface (int	 w,
 	       int	 h,
 	       GtkWidget *parent_style_window);
 
-GdkPixmap *
-create_native_pixmap_and_wrap (int	  w,
+cairo_surface_t *
+create_native_surface_and_wrap (int	  w,
 			       int	  h,
 			       GtkWidget *parent_style_window);
 
-GdkPixmap *
-pixmap_new_from_pixbuf (GdkPixbuf *pixbuf, GtkWidget *parent);
+cairo_surface_t *
+surface_new_from_pixbuf (GdkPixbuf *pixbuf, GtkWidget *parent);
 
 /* metacity.c */
+
 #ifdef USE_METACITY
 
 MetaFrameType
@@ -816,18 +627,6 @@ meta_get_frame_type_for_decor_type (const gchar *frame_type);
 void
 meta_draw_window_decoration (decor_t *d);
 
-void
-meta_get_decoration_geometry (decor_t		*d,
-			      MetaTheme	        *theme,
-			      MetaFrameFlags    *flags,
-			      MetaFrameGeometry *fgeom,
-			      MetaButtonLayout  *button_layout,
-			      MetaFrameType     frame_type,
-			      GdkRectangle      *clip);
-
-void
-meta_calc_button_size (decor_t *d);
-
 gboolean
 meta_calc_decoration_size (decor_t *d,
 			   gint    w,
@@ -846,10 +645,6 @@ meta_get_button_position (decor_t *d,
 			  gint    *w,
 			  gint    *h);
 
-gboolean
-meta_button_present (MetaButtonLayout   *button_layout,
-		     MetaButtonFunction function);
-
 void
 meta_get_event_window_position (decor_t *d,
 				gint    i,
@@ -874,6 +669,7 @@ void
 meta_get_shadow (decor_frame_t *, decor_shadow_options_t *opts, gboolean active);
 
 #endif
+
 /* switcher.c */
 
 #define SWITCHER_ALPHA 0xa0a0
@@ -896,34 +692,12 @@ create_switcher_frame (const gchar *);
 void
 destroy_switcher_frame ();
 
-void
-draw_switcher_decoration (decor_t *d);
-
 gboolean
 update_switcher_window (Window     popup,
 			Window     selected);
 
-decor_t *
-switcher_window_opened (Window popup, Window selected);
-
-void
-switcher_window_closed ();
-
 /* events.c */
 
-void
-move_resize_window (WnckWindow *win,
-		    int	       direction,
-		    decor_event *gtkwd_event);
-
-void
-common_button_event (WnckWindow *win,
-		     decor_event *gtkwd_event,
-		     decor_event_type gtkwd_type,
-		     int	button,
-		     int	max,
-		     char	*tooltip);
-
 void
 close_button_event (WnckWindow *win,
 		    decor_event *gtkwd_event,
@@ -973,26 +747,11 @@ unstick_button_event (WnckWindow *win,
 		      decor_event *gtkwd_event,
 		      decor_event_type gtkwd_type);
 
-void
-handle_title_button_event (WnckWindow   *win,
-			   int          action,
-			   decor_event *gtkwd_event);
-
-void
-handle_mouse_wheel_title_event (WnckWindow   *win,
-				unsigned int button);
-
 void
 title_event (WnckWindow       *win,
 	     decor_event      *gtkwd_event,
 	     decor_event_type gtkwd_type);
 
-void
-frame_common_event (WnckWindow       *win,
-		    int              direction,
-		    decor_event      *gtkwd_event,
-		    decor_event_type gtkwd_type);
-
 void
 top_left_event (WnckWindow       *win,
 		decor_event      *gtkwd_event,
@@ -1035,17 +794,6 @@ void
 frame_window_realized (GtkWidget *widget,
 		       gpointer  data);
 
-event_callback
-find_event_callback_for_point (decor_t *d,
-			       int     x,
-			       int     y,
-			       Bool    *enter,
-			       Bool    *leave,
-			       BoxPtr  *entered_box);
-
-event_callback
-find_leave_event_callback (decor_t *d);
-
 void
 frame_handle_button_press (GtkWidget      *widget,
 			   GdkEventButton *event,
@@ -1101,18 +849,6 @@ action_menu_map (WnckWindow *win,
 
 /* util.c */
 
-double
-square (double x);
-
-double
-dist (double x1, double y1,
-      double x2, double y2);
-
-void
-shade (const decor_color_t *a,
-       decor_color_t	   *b,
-       float		   k);
-
 gboolean
 get_window_prop (Window xwindow,
 		 Atom   atom,
@@ -1121,14 +857,8 @@ get_window_prop (Window xwindow,
 unsigned int
 get_mwm_prop (Window xwindow);
 
-
-/* style.c */
-
-void
-update_style (GtkWidget *widget);
-
-void
-style_changed (GtkWidget *widget, void *user_data /* PangoContext */);
+gchar *
+get_gtk_theme_variant (Window xwindow);
 
 /* settings.c */
 
